import { BehaviorSubject, of } from 'rxjs';
import { switchMap } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { ActivatedRoute, RouterModule } from '@angular/router';
import { Injectable, ChangeDetectionStrategy, Component, Input, NgModule, CUSTOM_ELEMENTS_SCHEMA, Inject, Optional, InjectionToken, ChangeDetectorRef, defineInjectable } from '@angular/core';
import { Title, DomSanitizer } from '@angular/platform-browser';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DocsService {
    constructor() {
        this._menuItems$ = new BehaviorSubject(null);
    }
    /**
     * @return {?}
     */
    get menuItems$() {
        return this._menuItems$.asObservable();
    }
    /**
     * @param {?} items
     * @return {?}
     */
    set menuItems(items) {
        this._menuItems = items;
        this._menuItems$.next(items);
    }
}
DocsService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ DocsService.ngInjectableDef = defineInjectable({ factory: function DocsService_Factory() { return new DocsService(); }, token: DocsService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const baseTitle = (title) => {
    return title.includes('|') ? title.split(' | ')[0] : title;
};
/** @type {?} */
const titleFromName = (name) => {
    return name.split(/(?=[A-Z])/).join(' ');
};
/** @type {?} */
const titleFromTag = (tag) => {
    return tag.split('-').slice(1).map((word) => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiComponent {
    /**
     * @param {?} title
     * @param {?} service
     */
    constructor(title, service) {
        this.title = title;
        this.service = service;
        this.service.menuItems = undefined;
        this.title.setTitle(`${baseTitle(this.title.getTitle())} | API`);
    }
}
ApiComponent.decorators = [
    { type: Component, args: [{
                template: '<router-outlet></router-outlet>',
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
ApiComponent.ctorParameters = () => [
    { type: Title },
    { type: DocsService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const FrameworkType = {
    angular: 'angular',
    javascript: 'javascript',
    vue: 'vue',
};
/** @type {?} */
const DEMO_COMPONENTS = new InjectionToken('Demo components');
/** @type {?} */
const GET_STARTED_URL = new InjectionToken('URL to get started from home');
/** @type {?} */
const HOME_README = new InjectionToken('Injectable Home Readme');
/** @type {?} */
const LIBRARY_API = new InjectionToken('The full library API');
/** @type {?} */
const MODULE_DOCS = new InjectionToken('Demo components');
/** @type {?} */
const MODULE_GET_STARTED_URL = new InjectionToken('URL to try a module');
/** @type {?} */
const MODULE_NAME = new InjectionToken('Module name');
/** @type {?} */
const MODULE_README = new InjectionToken('Injectable Web Components Readme');
/** @type {?} */
const MODULE_TYPE = new InjectionToken('The framework the module is for');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiDocComponent {
    /**
     * @param {?} title
     * @param {?} route
     * @param {?} api
     */
    constructor(title, route, api) {
        this.title = title;
        this.route = route;
        /** @type {?} */
        const module = this.route.snapshot.paramMap.get('module');
        this.id = this.route.snapshot.paramMap.get('id');
        this.doc = api[module].find(d => d.tag ? d.tag === this.id : d.name === this.id);
        this.title.setTitle(`${baseTitle(this.title.getTitle())} | ${this.id}`);
    }
}
ApiDocComponent.decorators = [
    { type: Component, args: [{
                template: "<article class=\"component-readme\">\n    <div class=\"breadcrumbs\">\n        <a [routerLink]=\"['../../']\">API</a><span class=\"sep\">/</span><span class=\"id\">{{id}}</span>\n    </div>\n    <ad-markdown [data]=\"doc.readme\"></ad-markdown>\n    <ad-api [doc]=\"doc\"></ad-api>\n</article>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host{display:block}:host .breadcrumbs{margin-bottom:1rem}:host .breadcrumbs .sep{color:var(--anj-border-dark);margin:0 .5rem}:host .breadcrumbs .id{color:var(--anj-neutral)}"]
            }] }
];
/** @nocollapse */
ApiDocComponent.ctorParameters = () => [
    { type: Title },
    { type: ActivatedRoute },
    { type: undefined, decorators: [{ type: Inject, args: [LIBRARY_API,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiListComponent {
    /**
     * @param {?} api
     */
    constructor(api) {
        this.api = api;
        this.filtered = this.api;
    }
    /**
     * @param {?} __0
     * @return {?}
     */
    filter({ detail }) {
        /** @type {?} */
        const query = detail.value.toLowerCase();
        /** @type {?} */
        const filtered = {};
        for (const module in this.api) {
            if (this.api.hasOwnProperty(module)) {
                /** @type {?} */
                const mFiltered = this.api[module].filter(d => this.nameFromDoc(d).toLowerCase().includes(query));
                if (mFiltered.length > 0) {
                    filtered[module] = mFiltered;
                }
            }
        }
        this.filtered = Object.keys(filtered).length > 0 ? filtered : undefined;
    }
    /**
     * @param {?} module
     * @param {?} doc
     * @return {?}
     */
    apiPath(module, doc) {
        return `${module}/${this.nameFromDoc(doc)}`;
    }
    /**
     * @param {?} doc
     * @return {?}
     */
    nameFromDoc(doc) {
        return doc.tag ? doc.tag : doc.name;
    }
}
ApiListComponent.decorators = [
    { type: Component, args: [{
                template: "<anj-form-group>\n    <anj-filter placeholder=\"Search\" (anjFilter)=\"filter($event)\"></anj-filter>\n</anj-form-group>\n<ng-container *ngIf=\"filtered; else noResults\">\n    <section *ngFor=\"let lib of filtered | keyvalue\">\n        <anj-headline><h3>{{lib.key}}</h3></anj-headline>\n        <div class=\"row results-container\">\n            <div *ngFor=\"let doc of lib.value\" class=\"col-12 col-sm-6 col-md-4 col-lg-3 result\">\n                <a [routerLink]=\"apiPath(lib.key, doc)\">{{nameFromDoc(doc)}}</a>\n            </div>\n        </div>\n    </section>\n</ng-container>\n<ng-template #noResults>\n    <p>No matches found.</p>\n</ng-template>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host{display:block}:host anj-headline{border-bottom:1px solid var(--anj-border);padding-bottom:.75rem;margin-bottom:1rem}:host .results-container{margin-bottom:2rem}:host .result{margin-bottom:.5rem}"]
            }] }
];
/** @nocollapse */
ApiListComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [LIBRARY_API,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ExternalDocComponent {
    /**
     * @param {?} docs
     * @param {?} route
     */
    constructor(docs, route) {
        this.docs = docs;
        this.route = route;
        this.doc$ = this.route.paramMap.pipe(switchMap((params) => of(this.docs[params.get('id')])));
    }
}
ExternalDocComponent.decorators = [
    { type: Component, args: [{
                template: '<ad-markdown [data]="doc$ | async"></ad-markdown>',
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
ExternalDocComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [MODULE_DOCS,] }] },
    { type: ActivatedRoute }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ExternalModuleComponent {
    /**
     * @param {?} docs
     * @param {?} name
     * @param {?} service
     */
    constructor(docs, name, service) {
        this.docs = docs;
        this.name = name;
        this.service = service;
        /** @type {?} */
        const path = name.toLowerCase();
        this.service.menuItems = Object.keys(docs).map((doc) => {
            return {
                link: `${path}/${doc}`,
                name: doc.split('-').join(' ')
            };
        });
    }
}
ExternalModuleComponent.decorators = [
    { type: Component, args: [{
                template: '<router-outlet></router-outlet>',
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
ExternalModuleComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [MODULE_DOCS,] }] },
    { type: String, decorators: [{ type: Inject, args: [MODULE_NAME,] }] },
    { type: DocsService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FeatureComponent {
    /**
     * @param {?} route
     * @param {?} changeRef
     * @param {?} title
     * @param {?} docs
     * @param {?} frameWorkType
     */
    constructor(route, changeRef, title, docs, frameWorkType) {
        this.route = route;
        this.changeRef = changeRef;
        this.title = title;
        this.docs = docs;
        this.frameWorkType = frameWorkType;
        this.route.params.subscribe({
            next: (params) => {
                /** @type {?} */
                const doc = this.docs.find((d) => {
                    return d.tag ? d.tag === params.id : d.name === params.id;
                });
                this.doc = this.filterExamples(doc);
                this.title.setTitle(this.titleFromDoc(this.title.getTitle()));
                this.changeRef.markForCheck();
            }
        });
    }
    /**
     * @param {?} usages
     * @return {?}
     */
    sortedKeys(usages) {
        /** @type {?} */
        const keys = Object.keys(usages);
        /** @type {?} */
        const firstIndex = keys.findIndex(k => k === 'usage' || k === 'angular' || k === 'vue');
        return keys.splice(firstIndex, 1).concat(keys);
    }
    /**
     * @private
     * @param {?} title
     * @return {?}
     */
    titleFromDoc(title) {
        /** @type {?} */
        const base = baseTitle(title);
        if (!this.doc) {
            return base;
        }
        return `${base} | ${this.doc.tag ? titleFromTag(this.doc.tag) : titleFromName(this.doc.name)}`;
    }
    /**
     * @private
     * @param {?} doc
     * @return {?}
     */
    filterExamples(doc) {
        if (this.frameWorkType === FrameworkType.angular) {
            return this.getNgExamples(doc);
        }
        else if (this.frameWorkType === FrameworkType.vue) {
            return this.getVueExamples(doc);
        }
        else if (this.frameWorkType === FrameworkType.javascript) {
            return this.getJsExamples(doc);
        }
        return doc;
    }
    // Return ng examples, if existing. Otherwise, js examples.
    /**
     * @param {?} doc
     * @return {?}
     */
    getNgExamples(doc) {
        /** @type {?} */
        const usages = {};
        if (doc.usage.angular) {
            for (const u in doc.usage) {
                if (u === 'angular' || u.startsWith('ng-')) {
                    usages[u] = doc.usage[u];
                }
            }
            doc.usage = usages;
            return doc;
        }
        else {
            return this.getJsExamples(doc);
        }
    }
    // Return js examples.
    /**
     * @param {?} doc
     * @return {?}
     */
    getJsExamples(doc) {
        /** @type {?} */
        const usages = {};
        for (const u in doc.usage) {
            if (u !== 'angular' && !u.startsWith('ng-') &&
                u !== 'vue' && !u.startsWith('vue-')) {
                usages[u] = doc.usage[u];
            }
        }
        doc.usage = usages;
        return doc;
    }
    // Return vue examples, if existing. Otherwise, js examples.
    /**
     * @param {?} doc
     * @return {?}
     */
    getVueExamples(doc) {
        /** @type {?} */
        const usages = {};
        if (doc.usage.vue) {
            for (const u in doc.usage) {
                if (u === 'vue' || u.startsWith('vue-')) {
                    usages[u] = doc.usage[u];
                }
            }
            doc.usage = usages;
            return doc;
        }
        else {
            return this.getJsExamples(doc);
        }
    }
}
FeatureComponent.decorators = [
    { type: Component, args: [{
                template: "<div class=\"row\">\n    <div [ngClass]=\"doc.usage ? 'col-12 col-lg-10' : 'col-12'\">\n        <article class=\"component-readme\" id=\"overview\">\n            <ad-markdown [data]=\"doc.readme\"></ad-markdown>\n        </article>\n        <article *ngFor=\"let key of sortedKeys(doc.usage)\" id=\"{{key}}\">\n            <ad-use-case [usage]=\"doc.usage[key]\"></ad-use-case>\n        </article>\n        <ad-api [doc]=\"doc\"></ad-api>\n    </div>\n    <div *ngIf=\"doc.usage\" class=\"d-none d-lg-flex col-lg-2 toc-wrapper\">\n        <ad-toc [docs]=\"doc.usage\" [keys]=\"sortedKeys(doc.usage)\"></ad-toc>\n    </div>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".component-readme{margin-bottom:2.5rem}"]
            }] }
];
/** @nocollapse */
FeatureComponent.ctorParameters = () => [
    { type: ActivatedRoute },
    { type: ChangeDetectorRef },
    { type: Title },
    { type: Array, decorators: [{ type: Inject, args: [MODULE_DOCS,] }] },
    { type: FrameworkType, decorators: [{ type: Optional }, { type: Inject, args: [MODULE_TYPE,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class HomeComponent {
    /**
     * @param {?} title
     * @param {?} service
     * @param {?} getStartedUrl
     * @param {?} readme
     */
    constructor(title, service, getStartedUrl, readme) {
        this.title = title;
        this.service = service;
        this.getStartedUrl = getStartedUrl;
        this.readme = readme;
        this.service.menuItems = null;
        this.title.setTitle(baseTitle(this.title.getTitle()));
    }
}
HomeComponent.decorators = [
    { type: Component, args: [{
                template: "<article>\n    <ad-markdown [data]=\"readme\"></ad-markdown>\n    <p *ngIf=\"getStartedUrl\" class=\"text-center pt-4\">\n        <anj-button [routerLink]=\"getStartedUrl\">Get Started</anj-button>\n    </p>\n</article>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
HomeComponent.ctorParameters = () => [
    { type: Title },
    { type: DocsService },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [GET_STARTED_URL,] }] },
    { type: String, decorators: [{ type: Inject, args: [HOME_README,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IntroComponent {
    /**
     * @param {?} title
     * @param {?} readme
     * @param {?} moduleName
     * @param {?} getStartedUrl
     */
    constructor(title, readme, moduleName, getStartedUrl) {
        this.title = title;
        this.readme = readme;
        this.moduleName = moduleName;
        this.getStartedUrl = getStartedUrl;
        this.title.setTitle(`${baseTitle(this.title.getTitle())} | ${moduleName}`);
    }
}
IntroComponent.decorators = [
    { type: Component, args: [{
                selector: 'ad-components',
                template: "<article>\n    <ad-markdown [data]=\"readme\"></ad-markdown>\n    <p *ngIf=\"getStartedUrl\" class=\"text-center pt-4\">\n        <anj-button [routerLink]=\"getStartedUrl\">Try It</anj-button>\n    </p>\n</article>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host::ng-deep ad-markdown p[align=center]{display:none}"]
            }] }
];
/** @nocollapse */
IntroComponent.ctorParameters = () => [
    { type: Title },
    { type: String, decorators: [{ type: Inject, args: [MODULE_README,] }] },
    { type: String, decorators: [{ type: Inject, args: [MODULE_NAME,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [MODULE_GET_STARTED_URL,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class JsDemo {
    /**
     * @protected
     * @param {?} element
     * @param {?} demo
     */
    constructor(element, demo) {
        element.nativeElement.appendChild(document.createRange().createContextualFragment(demo));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UseCaseComponent {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        /** @type {?} */
        const docs = this.usage.split('<!-- Demo -->');
        this.doc = docs[0].trim();
        this.demo = docs[1] ? this.sanitizer.bypassSecurityTrustHtml(docs[1] ? docs[1].trim() : '') : undefined;
        if (docs[2]) {
            /** @type {?} */
            const tabs = {};
            docs[2].trim().split('```').forEach((s) => {
                s = s.trim();
                if (s) {
                    tabs[s.split(/\n/)[0].trim()] = `${'```'}${s}${'```'}`;
                }
            });
            this.tabs = tabs;
        }
    }
    /**
     * @param {?=} tab
     * @return {?}
     */
    switchTabs(tab) {
        this.activeTab = tab ? tab : this.activeTab ? undefined : Object.keys(this.tabs)[0];
    }
    /**
     * @param {?} doc
     * @return {?}
     */
    getTitle(doc) {
        return doc.split('\n')[0].replace(/#/g, '').trim();
    }
    /**
     * @param {?} type
     * @return {?}
     */
    codeType(type) {
        return type.includes('typescript') ? 'ts' : type;
    }
}
UseCaseComponent.decorators = [
    { type: Component, args: [{
                selector: 'ad-use-case',
                template: "<ad-markdown [data]=\"doc\"></ad-markdown>\n<anj-card *ngIf=\"demo\" elevation=\"1\">\n    <anj-card-header>\n        <anj-headline>\n            <h3>{{getTitle(doc)}}</h3>\n            <anj-button context=\"icon\" (click)=\"switchTabs()\" title=\"View Source\">\n                <svg slot=\"icon-only\" xmlns=\"http://www.w3.org/2000/svg\" width=\"24\" height=\"24\" viewBox=\"0 0 24 24\">\n                    <path d=\"M9.4 16.6L4.8 12l4.6-4.6L8 6l-6 6 6 6 1.4-1.4zm5.2 0l4.6-4.6-4.6-4.6L16 6l6 6-6 6-1.4-1.4z\"/>\n                </svg>\n            </anj-button>\n        </anj-headline>\n    </anj-card-header>\n    <anj-card-body>\n        <div class=\"example-container\" [class.anj-active]=\"activeTab\">\n            <anj-tabs>\n                <anj-tab *ngFor=\"let t of tabs | keyvalue\" (click)=\"switchTabs(t.key)\" [class.anj-active]=\"t.key === activeTab\">{{codeType(t.key)}}</anj-tab>\n            </anj-tabs>\n            <div class=\"example\">\n                <ad-markdown *ngFor=\"let t of tabs | keyvalue\" [data]=\"t.value\" [class.anj-active]=\"t.key === activeTab\"></ad-markdown>\n            </div>\n        </div>\n        <div *ngIf=\"demo\" class=\"demo-container\" [innerHTML]=\"demo\"></div>\n    </anj-card-body>\n</anj-card>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host{border-bottom:1px solid var(--anj-border);display:block;margin-bottom:3rem;padding-bottom:3.5rem}:host anj-card{margin-top:2rem}:host anj-tab{text-transform:uppercase}:host .example-container{margin:0 -1.25rem}:host .example-container:not(.anj-active){height:0;overflow:hidden}:host .example{background-color:var(--anj-background);border-bottom:1px solid var(--anj-border);margin:0 0 1rem;padding:1rem}:host .example ad-markdown{display:none}:host .example ad-markdown.anj-active{display:block}"]
            }] }
];
/** @nocollapse */
UseCaseComponent.ctorParameters = () => [
    { type: DomSanitizer }
];
UseCaseComponent.propDecorators = {
    usage: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AnjunaDocsModule {
}
AnjunaDocsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    RouterModule
                ],
                declarations: [
                    ApiComponent,
                    ApiDocComponent,
                    ApiListComponent,
                    ExternalDocComponent,
                    ExternalModuleComponent,
                    FeatureComponent,
                    IntroComponent,
                    UseCaseComponent,
                    HomeComponent
                ],
                exports: [
                    ApiComponent,
                    ApiDocComponent,
                    ApiListComponent,
                    ExternalDocComponent,
                    ExternalModuleComponent,
                    FeatureComponent,
                    IntroComponent,
                    HomeComponent
                ],
                entryComponents: [
                    ApiComponent,
                    ApiDocComponent,
                    ApiListComponent,
                    FeatureComponent,
                    IntroComponent,
                    HomeComponent
                ],
                schemas: [
                    CUSTOM_ELEMENTS_SCHEMA
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiComponent, ApiDocComponent, ApiListComponent, ExternalDocComponent, ExternalModuleComponent, FeatureComponent, HomeComponent, IntroComponent, FrameworkType, DEMO_COMPONENTS, GET_STARTED_URL, HOME_README, LIBRARY_API, MODULE_DOCS, MODULE_GET_STARTED_URL, MODULE_NAME, MODULE_README, MODULE_TYPE, DocsService, baseTitle, titleFromName, titleFromTag, JsDemo, AnjunaDocsModule, UseCaseComponent as ɵa };

//# sourceMappingURL=anjuna-angular-docs.js.map